;-----------------------------------------------------------------------------
; Name: Window Example 02: Print (basic)
;
; File: print_ex01.asm
;
; Desc: Shows you how to print (to a printer)
;
; Source:   www.scrontsoft.com
;           scronty@scrontsoft.com
;
; Feel free to use this code anyway you see fit.
;-----------------------------------------------------------------------------

.386
.MODEL FLAT,STDCALL

OPTION SCOPED        ; Local labels are enabled, global labels inside
                     ; PROC should be defined with double colons (LABEL::)
OPTION CASEMAP:NONE  ; case sensitivity enabled
OPTION LJMP

MAIN_THREAD_STACKSIZE_KB=16

include     \masm32\include\windows.inc

include     \masm32\include\kernel32.inc
includelib  \masm32\Lib\kernel32.lib

include     \masm32\include\user32.inc
includelib  \masm32\Lib\user32.lib

include     \masm32\include\gdi32.inc
includelib  \masm32\Lib\gdi32.lib

include     \masm32\include\comdlg32.inc
includelib  \masm32\Lib\comdlg32.lib

include     \masm32\include\masm32.inc
includelib  \masm32\Lib\masm32.lib

PrintSetup PROTO :DWORD,:DWORD

.DATA
;-----------------------------------------------------------------------------
; Global variables
;-----------------------------------------------------------------------------
    hWindow                     HWND        0
    hInstance                   HANDLE      0
    hAccel                      HANDLE      0

    IDI_ICONp                   EQU         010h

    szIDPRINT                   db         "IDPRINT",0

    IDM_TEXT                    EQU         100
    IDM_EXIT                    EQU         101
    IDM_ABOUT                   EQU         102
    
    hbitmap                     HANDLE      0
    hFileMenu                   HANDLE      0
    
    MyClassName                 db "Window Example",0                     ; Our class name
    WinTitleString              db "Window Example 02: Print (basic)",0   ; Window title


    NUMLINES                    DWORD       20                  ; how many lines to print
    
    szStringToPrintonScreen     db "This is on the screen.",0   ; the string to print on window (obviously)
    strlengthScreen             DWORD       0                   ; length of window string

    szStringToPrinttoPrinter    db "This is on the printer.",0  ; the string to print on printer (obviously)
    strlengthPrinter            DWORD       0                   ; length of printer string
    
    szDocName                   db  "Name: Window Example 02: Print (basic)", 13, 10, 13, 10, 
                                    "Desc: Shows you how to print (to a printer)", 0

    szAbout                     db "About",0
    
    printdlg                    PRINTDLG    <>
    docinfo                     DOCINFO     <>

    X                           DWORD       0   ; current x output location
    Y                           DWORD       0   ; current y output location
    maxX                        DWORD       0   ; x screen dimensions
    maxY                        DWORD       0   ; y screen dimensions
    
    memDC                       HDC         0
    hBit                        HBITMAP     0
    hBrush                      HBRUSH      0
    
.CODE

;-----------------------------------------------------------------------------
; Name: MsgProc
; Desc: The window's message handler
;-----------------------------------------------------------------------------
MsgProc PROC hWnd:HWND,uMsg:UINT,wParam:WPARAM,lParam:LPARAM
LOCAL hdc           :DWORD
LOCAL ps            :PAINTSTRUCT
LOCAL response      :DWORD
LOCAL tm            :TEXTMETRIC
LOCAL copies        :DWORD
LOCAL buffer[128]   :BYTE
LOCAL l_width       :DWORD
LOCAL l_height      :DWORD

    .IF uMsg == WM_CREATE
        ; get screen coordinates
        INVOKE GetSystemMetrics, SM_CXSCREEN
        mov maxX, eax
        INVOKE GetSystemMetrics, SM_CYSCREEN
        mov maxY, eax

        ; create a virtual window
        INVOKE GetDC, hWnd
        mov hdc, eax
        INVOKE CreateCompatibleDC, hdc
        mov memDC, eax
        INVOKE CreateCompatibleBitmap, hdc, maxX, maxY
        mov hBit, eax
        INVOKE SelectObject, memDC, hBit
        INVOKE GetStockObject, WHITE_BRUSH
        mov hBrush, eax
        INVOKE SelectObject, memDC, hBrush
        INVOKE PatBlt, memDC, 0, 0, maxX, maxY, PATCOPY

        ; get text metrics
        INVOKE GetTextMetrics, hdc, ADDR tm

        INVOKE StrLen, ADDR szStringToPrintonScreen
        mov strlengthScreen, eax

        INVOKE StrLen, ADDR szStringToPrinttoPrinter
        mov strlengthPrinter, eax


        mov Y, 0
                    
        mov ecx, 0

@looptext:
        push ecx
        
        INVOKE TextOut, memDC, X, Y, ADDR szStringToPrintonScreen, strlengthScreen    ; output to memory
        INVOKE TextOut, hdc, X, Y, ADDR szStringToPrintonScreen, strlengthScreen      ; output to window

        ; advance to next line
        mov eax, Y
        add eax, tm.tmHeight
        add eax, tm.tmExternalLeading
        mov Y, eax

        pop ecx
        inc ecx
        cmp ecx, NUMLINES
jb @looptext

        INVOKE ReleaseDC, hWnd, hdc
        
    .ELSEIF uMsg == WM_COMMAND

        mov eax, wParam
        and eax, 0ffffh
        
        .IF eax == IDM_TEXT

;###################################################
        
            ; initialize the PRINTDLG structure
            INVOKE PrintSetup, ADDR printdlg, hWnd
       
            INVOKE PrintDlg, ADDR printdlg

            .IF eax != 0
                mov eax, sizeof DOCINFO
                mov docinfo.cbSize, eax
                mov eax, offset szDocName
                mov docinfo.lpszDocName, eax
                mov docinfo.lpszOutput, NULL
                mov docinfo.lpszDatatype, NULL
                mov docinfo.fwType, 0
                
                INVOKE GetTextMetrics, printdlg.hDC, ADDR tm

                INVOKE StartDoc, printdlg.hDC, ADDR docinfo

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; loop the number of copies requested
                mov ecx, 0

@printnumofcopiesloop:
                    push ecx

                    INVOKE StartPage, printdlg.hDC

                    mov Y, 0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; print a page
                    mov ecx, 0

@printnumoflinesloop:
                        push ecx

                        INVOKE TextOut, printdlg.hDC, X, Y, ADDR szStringToPrinttoPrinter, strlengthPrinter    ; output to printer

                        ; advance to next line
                        mov eax, Y
                        add eax, tm.tmHeight
                        add eax, tm.tmExternalLeading
                        mov Y, eax

                        pop ecx
                        inc ecx
                        cmp ecx, NUMLINES
jb @printnumoflinesloop

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

                    INVOKE EndPage, printdlg.hDC
                
                    pop ecx
                    inc ecx
                    cmp cx, printdlg.nCopies
jb @printnumofcopiesloop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                INVOKE EndDoc, printdlg.hDC
                INVOKE DeleteDC, printdlg.hDC

;###################################################

            .ENDIF
            
        .ELSEIF eax == IDM_ABOUT
            INVOKE MessageBox, hWnd, ADDR szDocName, ADDR szAbout, MB_OK
            
        .ELSEIF eax == IDM_EXIT
            INVOKE PostQuitMessage,0
            
        .ENDIF
        
    .ELSEIF uMsg == WM_PAINT
        INVOKE BeginPaint, hWnd, ADDR ps
        mov hdc, eax

        mov eax, ps.rcPaint.right
        sub eax, ps.rcPaint.left
        mov l_width, eax
        mov eax, ps.rcPaint.bottom
        sub eax, ps.rcPaint.top
        mov l_height, eax
        INVOKE BitBlt, hdc, ps.rcPaint.left, ps.rcPaint.top, l_width, l_height, memDC, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY

        INVOKE EndPaint, hWnd, ADDR ps

    .ELSEIF uMsg == WM_DESTROY

        INVOKE DeleteDC, memDC              ; delete the memory device
        INVOKE PostQuitMessage,0
        mov eax,0
        ret
        
    .ENDIF
        
    INVOKE DefWindowProc,hWnd,uMsg,wParam,lParam
    
    ret
MsgProc ENDP

;-----------------------------------------------------------------------------
; Name: WinMain
; Desc: The application's entry point
;-----------------------------------------------------------------------------
WinMain PROC hInst:DWORD,prevInstance:DWORD,cmdlinePtr:DWORD,cmdShow:DWORD
LOCAL wc:WNDCLASSEX
LOCAL msg:MSG
LOCAL rc:RECT
LOCAL pt:POINT
 
    mov eax,hInst
    mov hInstance, eax

    mov wc.cbSize, sizeof WNDCLASSEX 
    mov wc.style, CS_CLASSDC 
    mov wc.lpfnWndProc, MsgProc 
    mov wc.cbClsExtra, 0
    mov wc.cbWndExtra, 0
    mov wc.hInstance, eax 
    INVOKE LoadIcon, hInst, IDI_ICONp
    mov wc.hIcon,eax
    INVOKE LoadCursor, NULL, IDC_ARROW 
    mov wc.hCursor, eax  
    mov wc.hbrBackground, COLOR_WINDOWFRAME
    mov wc.lpszMenuName, offset szIDPRINT
    mov wc.lpszClassName, OFFSET MyClassName
    mov wc.hIconSm, NULL 

    INVOKE LoadMenu, hInst, ADDR szIDPRINT  ; Main menu ID
    mov hFileMenu, eax

    INVOKE RegisterClassEx,ADDR wc

;   Create the parent window
    INVOKE CreateWindowEx,NULL, ADDR MyClassName, ADDR WinTitleString, \
                              WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, 500, 400,\
                              0, NULL, hInstance, NULL
    mov hWindow, eax
    
;   Show the window
    INVOKE ShowWindow, hWindow, SW_SHOWDEFAULT
    INVOKE UpdateWindow, hWindow
    
    INVOKE LoadAccelerators, hInst, ADDR szIDPRINT
    mov hAccel, eax
    
      ;===================================
      ; Loop until PostQuitMessage is sent
      ;===================================
        .WHILE TRUE
            INVOKE GetMessage, ADDR msg, 0,0,0
		.BREAK .IF (!eax)
		INVOKE TranslateAccelerator, hWindow, hAccel, ADDR msg

            .IF	!eax
		    INVOKE TranslateMessage, ADDR msg
                INVOKE DispatchMessage, ADDR msg
		.ENDIF
        .ENDW
        
    INVOKE UnregisterClass,ADDR MyClassName,hInstance

    mov eax,msg.wParam
    ret
WinMain ENDP

PrintSetup PROC PSprintdlg:DWORD, hWnd:DWORD

    push edi
    mov edi, PSprintdlg
    
    mov eax, sizeof PRINTDLG
    mov [edi].PRINTDLG.lStructSize, eax
    mov eax, hWnd
    mov [edi].PRINTDLG.hWndOwner, eax
    mov [edi].PRINTDLG.hDevMode, NULL
    mov [edi].PRINTDLG.hDevNames, NULL
    mov [edi].PRINTDLG.hDC, NULL
    mov [edi].PRINTDLG.Flags, PD_RETURNDC OR PD_NOSELECTION OR PD_NOPAGENUMS
    mov [edi].PRINTDLG.nFromPage, 0
    mov [edi].PRINTDLG.nToPage, 0
    mov [edi].PRINTDLG.nMinPage, 0
    mov [edi].PRINTDLG.nMaxPage, 0
    mov [edi].PRINTDLG.nCopies, 1
    mov [edi].PRINTDLG.hInstance, NULL
    mov [edi].PRINTDLG.lCustData, 0
    mov [edi].PRINTDLG.lpfnPrintHook, NULL
    mov [edi].PRINTDLG.lpfnSetupHook, NULL
    mov [edi].PRINTDLG.lpPrintTemplateName, NULL
    mov [edi].PRINTDLG.lpSetupTemplateName, NULL
    mov [edi].PRINTDLG.hPrintTemplate, NULL
    mov [edi].PRINTDLG.hSetupTemplate, NULL

    pop edi
    
    ret
PrintSetup ENDP

main:
    INVOKE GetModuleHandle,NULL ; Get hInstance
    sub edx,edx
    INVOKE WinMain,eax,edx,edx,edx
    INVOKE ExitProcess,eax
END main
